@@ -56,7 +56,7 @@ BEGIN {
 
     @EXPORT = qw(setup cleanup shutdown check_url download get_unshare_cmd
     read_subuid_subgid CLONE_NEWNS CLONE_NEWUTS CLONE_NEWIPC CLONE_NEWUSER
-    CLONE_NEWPID CLONE_NEWNET test_unshare get_tar_compress_options);
+    CLONE_NEWPID CLONE_NEWNET PER_LINUX32 test_unshare get_tar_compress_options);
 
     $SIG{'INT'} = \&shutdown;
     $SIG{'TERM'} = \&shutdown;
@@ -162,6 +162,7 @@ sub check_url {
 
     # Load LWP::UserAgent if possible, else return 0.
     if (! can_load( modules => { 'LWP::UserAgent' => undef, } )) {
+	warn "install the libwww-perl package to support downloading dsc files";
 	return 0;
     }
 
@@ -302,7 +303,7 @@ sub download {
 
     # Print out amount of content received before returning the path of the
     # file.
-    print "Download of $url sucessful.\n";
+    print "Download of $url successful.\n";
     print "Size of content downloaded: ";
     if ($bytes_received >= 1024 * 1024) {
 	print sprintf("%.4g MB",
@@ -378,6 +379,11 @@ use constant {
     CLONE_NEWNET  => 0x40000000,
 };
 
+# from personality.h
+use constant {
+    PER_LINUX32 => 0x0008,
+};
+
 sub get_unshare_cmd($) {
     my $options = shift;
 
@@ -514,6 +520,11 @@ if (\$cpid != 0) {
 EOF
     }
 
+    if ($options->{'LINUX32'}) {
+        my $personality = PER_LINUX32;
+        $command .= "syscall &SYS_personality, $personality;";
+    }
+
     $command .= 'exec { $ARGV[0] } @ARGV or die "exec() failed: $!";';
     # remove code comments
     $command =~ s/^\s*#.*$//gm;
@@ -545,34 +556,28 @@ sub read_subuid_subgid() {
 	last if ($n eq $username);
     }
     close $fh;
-    push @result, ["u", 0, $subid, $num_subid];
 
-    if (scalar(@result) < 1) {
-	printf STDERR "/etc/subuid does not contain an entry for $username\n";
-	return;
-    }
-    if (scalar(@result) > 1) {
-	printf STDERR "/etc/subuid contains multiple entries for $username\n";
+    if ($n ne $username) {
+	printf STDERR "No entry for $username in /etc/subuid";
 	return;
     }
 
+    push @result, ["u", 0, $subid, $num_subid];
+
     open $fh, "<", "/etc/subgid" or die "cannot open /etc/subgid for reading: $!";
     while (my $line = <$fh>) {
 	($n, $subid, $num_subid) = split(/:/, $line, 3);
 	last if ($n eq $username);
     }
     close $fh;
-    push @result, ["g", 0, $subid, $num_subid];
 
-    if (scalar(@result) < 2) {
-	printf STDERR "/etc/subgid does not contain an entry for $username\n";
-	return;
-    }
-    if (scalar(@result) > 2) {
-	printf STDERR "/etc/subgid contains multiple entries for $username\n";
+    if ($n ne $username) {
+	printf STDERR "No entry for $username in /etc/subgid";
 	return;
     }
 
+    push @result, ["g", 0, $subid, $num_subid];
+
     return @result;
 }
 
@@ -627,6 +632,8 @@ sub get_tar_compress_options($) {
 	return ('--use-compress-program', 'lz4');
     } elsif ($filename =~ /\.(xz|txz)$/) {
 	return ('--xz');
+    } elsif ($filename =~ /\.zst$/) {
+	return ('--zstd');
     }
     return ();
 }
